!******************************************************************************!
!                                                                              !
!    class_loop.f90                                                            !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   class_loop.f90                                 |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.-N. Lang                             |            !
!        | `---' | UFO date: 23.11.2018                           |            !
!        |       | UFO version: 1.1                               |            !
!        |       | Generation date: 2019-10-09 13:25             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module class_loop
  use constants_mdl
  use input_mdl, only: error_mdl
  use lcSSVV0, only: loopSSVV0
  use lcFaFV0, only: loopFaFV0
  use lcSFFa2, only: loopSFFa2
  use lcUaUS0, only: loopUaUS0
  use lcFVFa1, only: loopFVFa1
  use lcUaVU0, only: loopUaVU0
  use lcSVS0, only: loopSVS0
  use lcFFaS1, only: loopFFaS1
  use lcSFaF1, only: loopSFaF1
  use lcUaSU0, only: loopUaSU0
  use lcVVSS0, only: loopVVSS0
  use lcFaFS0, only: loopFaFS0
  use lcFFaS2, only: loopFFaS2
  use lcSVSV0, only: loopSVSV0
  use lcSFFa0, only: loopSFFa0
  use lcFFaV0, only: loopFFaV0
  use lcSUaU0, only: loopSUaU0
  use lcFVFa0, only: loopFVFa0
  use lcUSUa0, only: loopUSUa0
  use lcFaSF0, only: loopFaSF0
  use lcFFaV1, only: loopFFaV1
  use lcFSFa1, only: loopFSFa1
  use lcFaSF2, only: loopFaSF2
  use lcSVVS0, only: loopSVVS0
  use lcVUUa0, only: loopVUUa0
  use lcVUaU0, only: loopVUaU0
  use lcSFFa1, only: loopSFFa1
  use lcVVS0, only: loopVVS0
  use lcVSV0, only: loopVSV0
  use lcSSV0, only: loopSSV0
  use lcSVV0, only: loopSVV0
  use lcVSSV0, only: loopVSSV0
  use lcVFaF0, only: loopVFaF0
  use lcSFaF2, only: loopSFaF2
  use lcUUaV0, only: loopUUaV0
  use lcFaFS1, only: loopFaFS1
  use lcVVVV0, only: loopVVVV0
  use lcVFFa1, only: loopVFFa1
  use lcUVUa0, only: loopUVUa0
  use lcVSVS0, only: loopVSVS0
  use lcFaVF1, only: loopFaVF1
  use lcVSS0, only: loopVSS0
  use lcSSS0, only: loopSSS0
  use lcSSSS0, only: loopSSSS0
  use lcFSFa2, only: loopFSFa2
  use lcUUaS0, only: loopUUaS0
  use lcVVV0, only: loopVVV0
  use lcSUUa0, only: loopSUUa0
  use lcVFFa0, only: loopVFFa0
  use lcSFaF0, only: loopSFaF0
  use lcFaVF0, only: loopFaVF0
  use lcFaSF1, only: loopFaSF1
  use lcFaFV1, only: loopFaFV1
  use lcVFaF1, only: loopVFaF1
  use lcUaUV0, only: loopUaUV0
  use lcFaFS2, only: loopFaFS2
  use lcFSFa0, only: loopFSFa0
  use lcFFaS0, only: loopFFaS0
  implicit none

  interface compute_loop_mdl
    module procedure computeLoop2, computeLoop3, computeLoop4, computeLoop5,   &
    computeLoop6, computeLoop7, computeLoop8
  end interface 

  contains

  subroutine computeLoop2(ty, co, wl, P1, PL1, m, wp, riMaxIn, riMaxOut,       &
  rankInc)
    use class_vertices
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop2

  subroutine computeLoop3(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn,    &
  riMaxOut, rankInc)
    use class_vertices
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case(FaFV0)
      call loopFaFV0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(SFFa2)
      call loopSFFa2(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(UaUS0)
      call loopUaUS0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FVFa1)
      call loopFVFa1(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(UaVU0)
      call loopUaVU0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(SVS0)
      call loopSVS0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut &
      , rankInc)
    case(FFaS1)
      call loopFFaS1(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(SFaF1)
      call loopSFaF1(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(UaSU0)
      call loopUaSU0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FaFS0)
      call loopFaFS0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FFaS2)
      call loopFFaS2(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(SFFa0)
      call loopSFFa0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FFaV0)
      call loopFFaV0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(SUaU0)
      call loopSUaU0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FVFa0)
      call loopFVFa0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(USUa0)
      call loopUSUa0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FaSF0)
      call loopFaSF0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FFaV1)
      call loopFFaV1(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FSFa1)
      call loopFSFa1(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FaSF2)
      call loopFaSF2(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(VUUa0)
      call loopVUUa0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(VUaU0)
      call loopVUaU0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(SFFa1)
      call loopSFFa1(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(VVS0)
      call loopVVS0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut &
      , rankInc)
    case(VSV0)
      call loopVSV0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut &
      , rankInc)
    case(SSV0)
      call loopSSV0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut &
      , rankInc)
    case(SVV0)
      call loopSVV0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut &
      , rankInc)
    case(VFaF0)
      call loopVFaF0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(SFaF2)
      call loopSFaF2(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(UUaV0)
      call loopUUaV0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FaFS1)
      call loopFaFS1(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(VFFa1)
      call loopVFFa1(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(UVUa0)
      call loopUVUa0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FaVF1)
      call loopFaVF1(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(VSS0)
      call loopVSS0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut &
      , rankInc)
    case(SSS0)
      call loopSSS0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut &
      , rankInc)
    case(FSFa2)
      call loopFSFa2(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(UUaS0)
      call loopUUaS0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(VVV0)
      call loopVVV0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut &
      , rankInc)
    case(SUUa0)
      call loopSUUa0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(VFFa0)
      call loopVFFa0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(SFaF0)
      call loopSFaF0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FaVF0)
      call loopFaVF0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FaSF1)
      call loopFaSF1(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FaFV1)
      call loopFaFV1(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(VFaF1)
      call loopVFaF1(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(UaUV0)
      call loopUaUV0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FaFS2)
      call loopFaFS2(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FSFa0)
      call loopFSFa0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case(FFaS0)
      call loopFFaS0(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn, riMaxOut&
      , rankInc)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop3

  subroutine computeLoop4(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp &
  , riMaxIn, riMaxOut, rankInc)
    use class_vertices
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
    complex (kind = dp), intent(in), dimension(0:3) :: w3
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case(SSVV0)
      call loopSSVV0(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp,     &
      riMaxIn, riMaxOut, rankInc)
    case(VVSS0)
      call loopVVSS0(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp,     &
      riMaxIn, riMaxOut, rankInc)
    case(SVSV0)
      call loopSVSV0(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp,     &
      riMaxIn, riMaxOut, rankInc)
    case(SVVS0)
      call loopSVVS0(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp,     &
      riMaxIn, riMaxOut, rankInc)
    case(VSSV0)
      call loopVSSV0(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp,     &
      riMaxIn, riMaxOut, rankInc)
    case(VVVV0)
      call loopVVVV0(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp,     &
      riMaxIn, riMaxOut, rankInc)
    case(VSVS0)
      call loopVSVS0(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp,     &
      riMaxIn, riMaxOut, rankInc)
    case(SSSS0)
      call loopSSSS0(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, m, wp,     &
      riMaxIn, riMaxOut, rankInc)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop4

  subroutine computeLoop5(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, w4, P4&
  , PL4, m, wp, riMaxIn, riMaxOut, rankInc)
    use class_vertices
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
    complex (kind = dp), intent(in), dimension(0:3) :: w3
    complex (kind = dp), intent(in), dimension(0:3) :: P4, PL4
    complex (kind = dp), intent(in), dimension(0:3) :: w4
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop5

  subroutine computeLoop6(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, w4, P4&
  , PL4, w5, P5, PL5, m, wp, riMaxIn, riMaxOut, rankInc)
    use class_vertices
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
    complex (kind = dp), intent(in), dimension(0:3) :: w3
    complex (kind = dp), intent(in), dimension(0:3) :: P4, PL4
    complex (kind = dp), intent(in), dimension(0:3) :: w4
    complex (kind = dp), intent(in), dimension(0:3) :: P5, PL5
    complex (kind = dp), intent(in), dimension(0:3) :: w5
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop6

  subroutine computeLoop7(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, w4, P4&
  , PL4, w5, P5, PL5, w6, P6, PL6, m, wp, riMaxIn, riMaxOut, rankInc)
    use class_vertices
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
    complex (kind = dp), intent(in), dimension(0:3) :: w3
    complex (kind = dp), intent(in), dimension(0:3) :: P4, PL4
    complex (kind = dp), intent(in), dimension(0:3) :: w4
    complex (kind = dp), intent(in), dimension(0:3) :: P5, PL5
    complex (kind = dp), intent(in), dimension(0:3) :: w5
    complex (kind = dp), intent(in), dimension(0:3) :: P6, PL6
    complex (kind = dp), intent(in), dimension(0:3) :: w6
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop7

  subroutine computeLoop8(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, w4, P4&
  , PL4, w5, P5, PL5, w6, P6, PL6, w7, P7, PL7, m, wp, riMaxIn, riMaxOut,      &
  rankInc)
    use class_vertices
    integer, intent(in) :: riMaxIn, riMaxOut, rankInc
    complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
    complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
    complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
    complex (kind = dp), intent(in), dimension(0:3) :: w2
    complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
    complex (kind = dp), intent(in), dimension(0:3) :: w3
    complex (kind = dp), intent(in), dimension(0:3) :: P4, PL4
    complex (kind = dp), intent(in), dimension(0:3) :: w4
    complex (kind = dp), intent(in), dimension(0:3) :: P5, PL5
    complex (kind = dp), intent(in), dimension(0:3) :: w5
    complex (kind = dp), intent(in), dimension(0:3) :: P6, PL6
    complex (kind = dp), intent(in), dimension(0:3) :: w6
    complex (kind = dp), intent(in), dimension(0:3) :: P7, PL7
    complex (kind = dp), intent(in), dimension(0:3) :: w7
    complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
    complex (kind = dp), intent(in) :: m
    complex (kind = dp), dimension(0:3) :: pl, p
    integer, intent(in) :: ty
    complex (kind = dp), dimension(0:), intent(in) :: co
    complex (kind = dp), dimension(0:3) :: sp1
    complex (kind = dp), dimension(0:3,0:3) :: sp2
    complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
    complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
    character(len=3) :: ty_str
    integer :: riOut, j
    logical :: isFirstRI
    integer :: mu, nu, ro, ta, te, tu
    select case(ty)
    case default
      write(ty_str,'(i3)') ty
      call error_mdl('Unhandled case: ' // ty_str, where='compute_loop_mdl')
    end select 
  end subroutine computeLoop8

end module class_loop
